import argparse
import tempfile
import os
from Bio.PDB import PDBParser, PDBIO, Select
from Bio.PDB.Atom import Atom
from Bio.PDB.Residue import Residue
from rdkit import Chem
from rdkit.Chem import AllChem

def load_palmitate_structure():
    """
    使用 RDKit 构建棕榈酸（C16H32O2）分子，并生成 PDB 格式的结构。
    """
    palm_mol = Chem.MolFromSmiles("CCCCCCCCCCCCCCCC(=O)O")  # 棕榈酸的 SMILES 表示
    palm_mol = Chem.AddHs(palm_mol)  # 添加氢原子
    AllChem.EmbedMolecule(palm_mol, randomSeed=42)  # 生成三维坐标
    AllChem.UFFOptimizeMolecule(palm_mol)  # 优化三维结构
    palm_pdb = Chem.MolToPDBBlock(palm_mol)  # 转换为 PDB 格式字符串
    
    # 创建临时文件来保存棕榈酸的 PDB 结构
    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        temp_file.write(palm_pdb.encode())
        temp_file.close()
        return temp_file.name

def add_palmitoylation_to_protein(protein_structure, residue_id, palm_pdb_path):
    """
    向蛋白质结构的指定残基添加棕榈酰化。
    """
    model = protein_structure[0]  # 使用第一个模型
    chain_name = 'A'  # 假设目标链是 A
    residue_id = (" ", int(residue_id), " ")  # 格式化残基编号

    try:
        chain = model[chain_name]  # 获取链 A
        residue = chain[residue_id]  # 获取残基编号
    except KeyError:
        print(f"Error: Chain {chain_name} or residue {residue_id} not found.")
        print("Available chains and residues:")
        for chain in model:
            print(f"Chain {chain.id}")
            for res in chain:
                print(f"Residue: {res.get_resname()} {res.get_id()}")
        return None

    print(f"Target residue found: {residue.resname} {residue.id}")

    # 解析棕榈酸的 PDB 文件
    palm_structure = PDBParser(QUIET=True).get_structure("PALM", palm_pdb_path)
    palm_model = palm_structure[0]
    palm_residue = list(palm_model.get_residues())[0]  # 棕榈酸作为单一残基

    # 获取目标残基的 SG 原子位置
    if "SG" not in residue:
        print(f"Error: Residue {residue.resname} {residue.id} does not contain an SG atom.")
        return None

    sg_atom = residue["SG"]  # 硫原子
    sg_coord = sg_atom.coord

    # 添加棕榈酸原子
    for atom in palm_residue:
        new_coord = atom.coord + sg_coord  # 平移棕榈酸以连接到 SG 原子
        new_atom = Atom(
            atom.name,
            new_coord,
            atom.bfactor,
            atom.occupancy,
            atom.altloc,
            atom.fullname,
            atom.serial_number,
            atom.element,
        )
        residue.add(new_atom)

    print("Palmitoylation added successfully.")
    return protein_structure

def main():
    parser = argparse.ArgumentParser(description="向蛋白质的指定残基添加棕榈酰化。")
    parser.add_argument("input_pdb", help="输入的 PDB 文件路径")
    parser.add_argument("residue_id", type=int, help="目标残基编号")
    parser.add_argument("output_pdb", help="输出的 PDB 文件路径")
    args = parser.parse_args()

    # 解析输入的 PDB 文件
    pdb_parser = PDBParser(QUIET=True)
    protein_structure = pdb_parser.get_structure("PROTEIN", args.input_pdb)

    # 生成棕榈酸的临时 PDB 结构文件
    palm_pdb_path = load_palmitate_structure()

    # 添加棕榈酰化
    modified_structure = add_palmitoylation_to_protein(
        protein_structure, args.residue_id, palm_pdb_path
    )

    if modified_structure is not None:
        # 保存修改后的 PDB 文件
        io = PDBIO()
        io.set_structure(modified_structure)
        io.save(args.output_pdb)
        print(f"Modified PDB saved to {args.output_pdb}")
    else:
        print("Failed to modify the protein structure.")

    # 删除临时文件
    os.remove(palm_pdb_path)

if __name__ == "__main__":
    main()
